# -*- coding: utf-8 -*-
"""
Created on Mon Feb 13 22:27:25 2023

@author: z0039mte
"""

import numpy as np

import tensortrax as tr
import tensortrax.math as tm


def psi(F, mu=1):
    C = tm.dot(tm.transpose(F), F)
    I1 = tm.trace(C)
    return mu * (I1 - 3) / 2


def neo_hooke_ext(F):
    J = tm.linalg.det(F)
    return tm.external(
        x=J ** (-1 / 3) * F,
        function=tr.function(psi, ntrax=F.ntrax),
        gradient=tr.gradient(psi, ntrax=F.ntrax),
        hessian=tr.hessian(psi, ntrax=F.ntrax),
        indices="ij",
    )


def neo_hooke(F, mu=1):
    C = tm.dot(tm.transpose(F), F)
    I1 = tm.trace(C)
    J = tm.linalg.det(F)
    return mu * (J ** (-2 / 3) * I1 - 3) / 2


def test_external():
    F = np.tile((np.eye(3).ravel() + np.arange(9) / 10).reshape(3, 3, 1, 1), 2100)

    W = []
    dWdF = []
    d2WdF2 = []

    for fun in [neo_hooke, neo_hooke_ext]:
        W.append(tr.function(fun, ntrax=2)(F))
        dWdF.append(tr.gradient(fun, ntrax=2)(F))
        d2WdF2.append(tr.hessian(fun, wrt="F", ntrax=2)(F=F))

        assert W[-1].shape == (1, 2100)
        assert dWdF[-1].shape == (3, 3, 1, 2100)
        assert d2WdF2[-1].shape == (3, 3, 3, 3, 1, 2100)

    assert np.allclose(*W)
    assert np.allclose(*dWdF)
    assert np.allclose(*d2WdF2)

    W = tm.external(
        x=F,
        function=neo_hooke,
        gradient=None,
        hessian=None,
        indices=None,
    )

    assert W.shape == (1, 2100)


if __name__ == "__main__":
    test_external()



r"""
 _
| |                          ████████╗██████╗  █████╗ ██╗  ██╗
| |_ ___ _ __  ___  ___  _ __╚══██╔══╝██╔══██╗██╔══██╗╚██╗██╔╝
| __/ _ \ '_ \/ __|/ _ \| '__|  ██║   ██████╔╝███████║ ╚███╔╝
| ||  __/ | | \__ \ (_) | |     ██║   ██╔══██╗██╔══██║ ██╔██╗
 \__\___|_| |_|___/\___/|_|     ██║   ██║  ██║██║  ██║██╔╝ ██╗
                                ╚═╝   ╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝
"""

from .._tensor import broadcast_to, dual2real, ravel, reshape, squeeze
from . import _linalg as linalg
from . import _math_array as base
from . import _special as special
from ._math_tensor import (
    abs,
    array,
    cos,
    cosh,
    diagonal,
    dot,
    einsum,
    exp,
    external,
    hstack,
    if_else,
    log,
    log10,
    matmul,
    repeat,
    sign,
    sin,
    sinh,
    split,
    sqrt,
    stack,
    sum,
    tan,
    tanh,
    tile,
    trace,
    transpose,
    vstack,
)

__all__ = [
    "linalg",
    "base",
    "special",
    "abs",
    "array",
    "broadcast_to",
    "cos",
    "cosh",
    "diagonal",
    "dot",
    "dual2real",
    "einsum",
    "exp",
    "external",
    "hstack",
    "log",
    "log10",
    "matmul",
    "ravel",
    "repeat",
    "reshape",
    "sign",
    "sin",
    "sinh",
    "split",
    "sqrt",
    "squeeze",
    "stack",
    "sum",
    "tan",
    "tanh",
    "tile",
    "trace",
    "transpose",
    "vstack",
    "if_else",
]
